#ifndef TESTIMAGES_H_INCLUDED
#define TESTIMAGES_H_INCLUDED

#include "misc.h" // for Lerp

inline int IsInsideCircle(const Vector2 p, const float radius)
{
	return (p.getLengthSquared() < radius*radius) ? 1 : 0;
}

inline int IsBelowHorizon(const Vector2 p)
{
	return (p.y >= 0) ? 1 : 0;
}


inline int SampleTestImageAt(const unsigned short whichImage, Vector2 samplePoint)
{
	bool isInside;

	// small circle
	if (whichImage == 1 || whichImage == 2 || whichImage == 3 || whichImage == 4)
	{
		const float radius = 100;

		Vector2 offset(0,0);
		if (whichImage == 2)
			offset = Vector2(50,0);
		else if (whichImage == 3)
			offset = Vector2(100,0);
		else if (whichImage == 4)
			offset = Vector2(150,0);
		samplePoint = samplePoint - offset;

		isInside = IsInsideCircle(samplePoint, radius);
	}

	// big circle
	else if (whichImage == 5 || whichImage == 6 || whichImage == 7 || whichImage == 8)
	{
		const float radius = 400;

		Vector2 offset(0,0);
		if (whichImage == 6)
			offset = Vector2(300,0);
		else if (whichImage == 7)
			offset = Vector2(400,0);
		else if (whichImage == 8)
			offset = Vector2(500,0);
		samplePoint = samplePoint - offset;

		isInside = IsInsideCircle(samplePoint, radius);
	}

	// vertical dividing line
	else if (whichImage == 9 || whichImage == 10)
	{
		Vector2 offset(0,0);
		if (whichImage == 10)
			offset = Vector2(100,0);
		samplePoint = samplePoint - offset;

		samplePoint.rotateBy90();
		isInside = IsBelowHorizon(samplePoint);
	}

	// straight angle, 90 degrees
	else if (whichImage == 11 || whichImage == 12 || whichImage == 13)
	{
		Vector2 offset(0,0);
		if (whichImage == 11)
			offset = Vector2(-100,0);
		if (whichImage == 13)
			offset = Vector2(100,0);
		samplePoint = samplePoint - offset;

		isInside =
		(
			IsBelowHorizon(Vector2::rotatedBy90(samplePoint))
			&&
			IsBelowHorizon(samplePoint)
		);
	}

	// blunt angle, 135 degrees
	else if (whichImage >= 14 && whichImage <= 17)
	{
		Vector2 offset(0, 0);
		if (whichImage == 14)
			offset.set(-100, 0);
		if (whichImage == 16)
			offset.set(100, 0);
		if (whichImage == 17)
			offset.set(-100, 100);
		if (whichImage == 18)
			offset.set(0, 100);
		if (whichImage == 19)
			offset.set(100, 100);
		samplePoint = samplePoint - offset;

		isInside =
		(
			IsBelowHorizon(Vector2::rotatedBy90(samplePoint))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*0.25))
		);
	}

	// blank background
	else if (whichImage == 18)
	{
		isInside = true;
	}

	// two straight angles, placed on opposing diagonals
	// can also be regarded as image 31 with half the boxes missing
	else if (whichImage == 19)
	{
		isInside =
		(
			(
				IsBelowHorizon(Vector2::rotatedBy180(samplePoint) + Vector2(0, 100))
				||
				IsBelowHorizon(Vector2::rotatedBy270(samplePoint) + Vector2(0, 100))
			)
			&&
			(
				IsBelowHorizon(                      samplePoint  + Vector2(0, 100))
				||
				IsBelowHorizon(Vector2::rotatedBy90 (samplePoint) + Vector2(0, 100))
			)
		);
	}

	// sharp angle, 45 degrees
	else if (whichImage >= 20 && whichImage <= 22)
	{
		Vector2 offset(0, 0);
		if (whichImage == 20)
			offset.set(-100, 0);
		if (whichImage == 22)
			offset.set(100, 0);
		samplePoint = samplePoint - offset;

		isInside =
		(
			IsBelowHorizon(Vector2::rotatedBy90(samplePoint))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*0.25))
		);
	}







	// vertical dividing line again, at other offsets
	else if (whichImage == 23 || whichImage == 24)
	{
		Vector2 offset(0, 0);
		if (whichImage == 23)
			offset = Vector2(50,0);
		if (whichImage == 24)
			offset = Vector2(200,0);
		samplePoint = samplePoint - offset;

		samplePoint.rotateBy90();
		isInside = (IsBelowHorizon(samplePoint));
	}

	// dividing line at various angles
	else if (whichImage >= 25 && whichImage <= 26)
	{
		float angle;
		if (whichImage == 25)
			angle = M_PI*0.5*0.5;
		if (whichImage == 26)
			angle = 0;

		if (angle)
			samplePoint.rotateBy(angle);

		isInside = (IsBelowHorizon(samplePoint));
	}

	// circles again, at various sizes
	else if (whichImage == 27)
	{
		const float radius = 50;
		isInside = (IsInsideCircle(samplePoint, radius));
	}
	else if (whichImage == 28)
	{
		const float radius = 200;
		isInside = (IsInsideCircle(samplePoint, radius));
	}

	// two vertical dividing lines
	else if (whichImage == 29)
	{
		isInside =
		(
			IsBelowHorizon(Vector2::rotatedBy90 (samplePoint) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy270(samplePoint) + Vector2(0, 100))
		);
	}

	// square, 50 pixels from center to sides
	else if (whichImage == 30)
	{
		isInside =
		(
			IsBelowHorizon(                      samplePoint  + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy180(samplePoint) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy90 (samplePoint) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy270(samplePoint) + Vector2(0, 100))
		);
		// + Vector2(0, 70.71) would give 100 pixels from center to corner points
	}

	// "shadow" of square, like a flag
	else if (whichImage == 31)
	{
		isInside =
		(
			(
				IsBelowHorizon(                      samplePoint  + Vector2(0, 100))
				&&
				IsBelowHorizon(Vector2::rotatedBy180(samplePoint) + Vector2(0, 100))
			)
			||
			(
				IsBelowHorizon(Vector2::rotatedBy90 (samplePoint) + Vector2(0, 100))
				&&
				IsBelowHorizon(Vector2::rotatedBy270(samplePoint) + Vector2(0, 100))
			)
		);
	}

	/*
	// equilateral triangle, 100 pixels from center to corner points
	+ Vector2(0, 31,622778)
	*/

	// equilateral triangle, 50 pixels from center to sides
	else if (whichImage == 32)
	{
		isInside =
		(
			IsBelowHorizon(Vector2::rotatedBy180(samplePoint) + Vector2(0, 50))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*0.333333) + Vector2(0, 50))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*0.333333) + Vector2(0, 50))
		);
	}
	// equilateral triangle "shadow"
	else if (whichImage == 33)
	{
		bool a = !IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*0.333333) + Vector2(0, 50));
		bool b = !IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*0.333333) + Vector2(0, 50));
		bool c = !IsBelowHorizon(Vector2::rotatedBy180(samplePoint) + Vector2(0, 50));
		isInside =
		(
			(a && b)
			||
			(a && c)
			||
			(b && c)
		);
	}

	// hexagon, 50 pixels from center to sides
	else if (whichImage == 34)
	{
		isInside =
		(
			IsBelowHorizon(                      samplePoint  + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy180(samplePoint) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*0.333333) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*0.333333) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*0.6666667) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*0.6666667) + Vector2(0, 100))
		);
	}

	// octagon, 50 pixels from center to sides
	else if (whichImage == 35)
	{
		isInside =
		(
			IsBelowHorizon(                      samplePoint  + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy180(samplePoint) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy90 (samplePoint) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy270(samplePoint) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*0.25) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*0.25) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*0.75) + Vector2(0, 100))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*0.75) + Vector2(0, 100))
		);
	}

	// two lines meeting at various angles, symmetric around vertical axis
	else if (whichImage >= 36 && whichImage <= 39)
	{
		float angle = 0;
		if (whichImage == 36)
			angle = 45/4;
		if (whichImage == 37)
			angle = 90/2;
		if (whichImage == 38)
			angle = 120/2;
		if (whichImage == 39)
			angle = 157.5/2;

		isInside =
		(
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*angle/180))
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*angle/180))
		);
	}

	// 60 degree angle symmetric around vertical axis, with different vertical positioning
	else if (whichImage >= 40 && whichImage <= 41)
	{
		Vector2 offset(0, 0);
		if (whichImage == 40)
			offset = Vector2(0,-50);
		if (whichImage == 41)
			offset = Vector2(0, 50);
		isInside =
		(
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*60/180) + offset)
			&&
			IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*60/180) + offset)
		);
	}

	// sharp corner becoming more and more round
	else if (whichImage >= 42 && whichImage <= 46)
	{
		float radius = 0;
		if (whichImage == 42)
			radius = 20;
		else if (whichImage == 43)
			radius = 50;
		else if (whichImage == 44)
			radius = 100;
		else if (whichImage == 45)
			radius = 200;
		else if (whichImage == 46)
			radius = 400;

		Vector2 offset(0, radius*0.41421);
		samplePoint = samplePoint + offset;

		isInside =
		(
			(
				IsBelowHorizon(Vector2::rotatedBy(samplePoint, M_PI*45/180))
				&&
				IsBelowHorizon(Vector2::rotatedBy(samplePoint, -M_PI*45/180))
				&&
				IsBelowHorizon(samplePoint-Vector2(0,radius/1.41421))
			)
			||
			IsInsideCircle(samplePoint-Vector2(0,radius*1.41421), radius)
		);
	}

	// tree branches, comes from an image file, too complex to generate here
	// else if (whichImage == 47)

	return (isInside ? 1 : 0);
}


void DrawTestImage(unsigned short whichImage, ALLEGRO_COLOR foreground, ALLEGRO_COLOR background, Vector2 pos, Vector2 crop, float zoom)
{
	//al_clear_to_color(background);

	ALLEGRO_BITMAP* bmp = al_get_target_bitmap();
	bool handleLockingHere = !(al_is_bitmap_locked(bmp));
	if (handleLockingHere)
		// gotta lock the target bitmap, otherwise al_put_pixel is sloooooooow!
		al_lock_bitmap(bmp, ALLEGRO_PIXEL_FORMAT_ANY, 0);

	unsigned char foreground_r;
	unsigned char foreground_g;
	unsigned char foreground_b;
	al_unmap_rgb(foreground, &foreground_r, &foreground_g, &foreground_b);

	unsigned char background_r;
	unsigned char background_g;
	unsigned char background_b;
	al_unmap_rgb(background, &background_r, &background_g, &background_b);

	const int nSamples = 2;
	ALLEGRO_COLOR mixcol[nSamples*nSamples+1];
	for (int i = 0; i <= nSamples*nSamples; i++)
	{
		if (i == 0)
			mixcol[i] = background;
		else if (i == nSamples*nSamples)
			mixcol[i] = foreground;
		else
		{
			float alpha = (float)i / (nSamples*nSamples);
			mixcol[i] = al_map_rgb(
							Lerp(background_r, foreground_r, alpha),
							Lerp(background_g, foreground_g, alpha),
							Lerp(background_b, foreground_b, alpha));
		}
	}

	for (int scrPY = pos.y-crop.y/2; scrPY < pos.y+crop.y/2; scrPY++)
	{
		for (int scrPX = pos.x-crop.x/2; scrPX < pos.x+crop.x/2; scrPX++)
		{
			Vector2 samplePoint(scrPX - pos.x, scrPY - pos.y);

			int isInside = 0;
			Vector2i ms(0, 0);
			for (ms.y = 0; ms.y < nSamples; ms.y++)
			{
				for (ms.x = 0; ms.x < nSamples; ms.x++)
				{
					Vector2 asdf(ms.x, ms.y);
					asdf /= nSamples;
					isInside += SampleTestImageAt(whichImage, (samplePoint+asdf)/zoom);
				}
			}
			// TODO: optimize, begin with single sampling, then only do multisampling along the edges
			// (if any 8-neighbour has a different inside/outside value than me)

			if (isInside)
			{
				al_put_pixel(scrPX, scrPY, mixcol[isInside]);
			}
		}
	}

	if (handleLockingHere)
		al_unlock_bitmap(bmp);
}

/*
This old function uses explicit drawing commands.
It worked fine, but I decided it's better to define analytical shapes as in the new function,
because there we can do cartesian-polar coordinate transforms before sampling.
void DrawTestImage_old(unsigned short whichImage, ALLEGRO_COLOR foreground, ALLEGRO_COLOR background, Vector2 pos, Vector2 crop, float zoom)
{
	// preconditions:
	// zoom > 0
	// 100*zoom < crop.x/2
	// 100*zoom < crop.y/2

	unsigned char foreground_r;
	unsigned char foreground_g;
	unsigned char foreground_b;
	al_unmap_rgb(foreground, &foreground_r, &foreground_g, &foreground_b);

	al_clear_to_color(background);

	//zoom = size.x / gameW;
	//zoom = size.y / gameH;
	//size.x = zoom * gameW;
	//size.y = zoom * gameH;

	// small circle
	if (whichImage == 1)
	{
		//al_draw_filled_circle(gameW/2, gameH/2, 100*zoom, foreground);
		FillCircleAntiAliased(pos.x, pos.y, 100*zoom, foreground, background);
	}
	else if (whichImage == 2)
	{
		//al_draw_filled_circle(gameW/2 + 50*zoom, gameH/2, 100*zoom, foreground);
		FillCircleAntiAliased(pos.x + 50*zoom, pos.y, 100*zoom, foreground, background);
	}
	else if (whichImage == 3)
	{
		//al_draw_filled_circle(gameW/2 + 100*zoom, gameH/2, 100*zoom, foreground);
		FillCircleAntiAliased(pos.x + 100*zoom, pos.y, 100*zoom, foreground, background);
	}
	else if (whichImage == 4)
	{
		//al_draw_filled_circle(gameW/2 + 150*zoom, gameH/2, 100*zoom, foreground);
		FillCircleAntiAliased(pos.x + 150*zoom, pos.y, 100*zoom, foreground, background);
	}

	// big circle
	else if (whichImage == 5)
	{
		//al_draw_filled_circle(gameW/2, gameH/2, 400*zoom, foreground);
		FillCircleAntiAliased(pos.x, pos.y, 400*zoom, foreground, background);
	}
	else if (whichImage == 6)
	{
		//al_draw_filled_circle(gameW/2 + 300*zoom, gameH/2, 400*zoom, foreground);
		FillCircleAntiAliased(pos.x + 300*zoom, pos.y, 400*zoom, foreground, background);
	}
	else if (whichImage == 7)
	{
		//al_draw_filled_circle(gameW/2 + 400*zoom, gameH/2, 400*zoom, foreground);
		FillCircleAntiAliased(pos.x + 400*zoom, pos.y, 400*zoom, foreground, background);
	}
	else if (whichImage == 8)
	{
		//al_draw_filled_circle(gameW/2 + 500*zoom, gameH/2, 400*zoom, foreground);
		FillCircleAntiAliased(pos.x + 500*zoom, pos.y, 400*zoom, foreground, background);
	}

	// straight dividing line
	else if (whichImage == 9)
	{
		al_draw_filled_rectangle(pos.x, pos.y-crop.y/2, pos.x+crop.x/2, pos.y+crop.y/2, foreground);
		//Vector2 p0(gameW/2, 0);
		//Vector2 p1(gameW/2, gameH);
		//DrawRibbonAntiAliased(p0, p1, foreground, 3);
	}
	else if (whichImage == 10)
	{
		al_draw_filled_rectangle(pos.x + 100*zoom, pos.y-crop.y/2, pos.x+crop.x/2, pos.y+crop.y/2, foreground);
		//Vector2 p0(gameW/2 + 100, 0);
		//Vector2 p1(gameW/2 + 100, gameH);
		//DrawRibbonAntiAliased(p0, p1, foreground, 3);
	}

	// straight angle
	else if (whichImage == 11)
	{
		al_draw_filled_rectangle(pos.x - 100*zoom, pos.y, pos.x+crop.x/2, pos.y+crop.y/2, foreground);
		//Vector2 p0(gameW/2 - 100, gameH);
		//Vector2 p1(gameW/2 - 100, gameH/2);
		//Vector2 p2(gameW, gameH/2);
		// TODO: FIX SERIOUS CRASH BUG!! seems non-deterministic...
		//DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 12)
	{
		al_draw_filled_rectangle(pos.x, pos.y, pos.x+crop.x/2, pos.y+crop.y/2, foreground);
		//Vector2 p0(gameW/2, gameH);
		//Vector2 p1(gameW/2, gameH/2);
		//Vector2 p2(gameW, gameH/2);
		//DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 13)
	{
		al_draw_filled_rectangle(pos.x + 100*zoom, pos.y, pos.x+crop.x/2, pos.y+crop.y/2, foreground);
		//Vector2 p0(gameW/2 + 100, gameH);
		//Vector2 p1(gameW/2 + 100, gameH/2);
		//Vector2 p2(gameW, gameH/2);
		//DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}

	// blunt angle
	else if (whichImage == 14)
	{
		Vector2 wholeImageOffset(0, 0);
		if (whichImage == 14)
			wholeImageOffset.set(-100, 0);
		if (whichImage == 16)
			wholeImageOffset.set(100, 0);
		if (whichImage == 17)
			wholeImageOffset.set(-100, 100);
		if (whichImage == 18)
			wholeImageOffset.set(0, 100);
		if (whichImage == 19)
			wholeImageOffset.set(100, 100);
		wholeImageOffset *= zoom;

		Vector2 p0(0, 0);
		Vector2 p1(0, 1);
		Vector2 p2(1, -1);
		//Vector2 crop(gameW, gameH);
		Vector2 lineOrigin = crop/2 + wholeImageOffset;
		p0 = lineOrigin;
		p1.scaleToScreenEdge(lineOrigin, crop);
		p2.scaleToScreenEdge(lineOrigin, crop);
		p0 += pos - crop/2;
		p1 += pos - crop/2;
		p2 += pos - crop/2;

		al_draw_filled_rectangle(p0.x, p2.y, gameW, gameH, foreground);
		al_draw_filled_triangle(p0.x, p0.y, p0.x, p2.y, p2.x, p2.y, background);

		// make the 45 degree line anti-aliased
		ALLEGRO_COLOR forehalf = al_map_rgb(foreground_r/2, foreground_g/2, foreground_b/2); // TODO: gamma correct
		al_lock_bitmap(al_get_target_bitmap(), ALLEGRO_PIXEL_FORMAT_ANY, 0);
		for (int x = p0.x; x < p2.x; x++)
			al_put_pixel(x, p0.y - (x-p0.x) - 1, forehalf);
		al_unlock_bitmap(al_get_target_bitmap());

//		Vector2 p0(gameW/2 - 100, gameH);
//		Vector2 p1(gameW/2 - 100, gameH/2);
//		Vector2 p2(gameW/2+gameH/2 - 100, 0);
//		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 15)
	{
		//al_draw_filled_rectangle(gameW/2, gameH/2, gameW, gameH, foreground);
		//al_draw_filled_triangle(gameW/2, gameH/2, gameW, gameH/2, gameW, 0, foreground);
//		Vector2 p0(gameW/2, gameH);
//		Vector2 p1(gameW/2, gameH/2);
//		Vector2 p2(gameW/2+gameH/2, 0);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 16)
	{
		//al_draw_filled_rectangle(gameW/2 + 100, gameH/2, gameW, gameH, foreground);
		//al_draw_filled_triangle(gameW/2 + 100, gameH/2, gameW + 100, gameH/2, gameW + 100, 0, foreground);
//		Vector2 p0(gameW/2 + 100, gameH);
//		Vector2 p1(gameW/2 + 100, gameH/2);
//		Vector2 p2(gameW/2+gameH/2 + 100, 0);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 17)
	{
		//al_draw_filled_rectangle(gameW/2 - 100, gameH/2 + 100, gameW, gameH, foreground);
		//al_draw_filled_triangle(gameW/2 - 100, gameH/2 + 100, gameW, gameH/2 + 100, gameW, 0, foreground);
		Vector2 p0(gameW/2 - 100, gameH + 100);
		Vector2 p1(gameW/2 - 100, gameH/2 + 100);
		Vector2 p2(gameW/2+gameH/2, 0);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 18)
	{
		//al_draw_filled_rectangle(gameW/2, gameH/2 + 100, gameW, gameH, foreground);
		//al_draw_filled_triangle(gameW/2, gameH/2 + 100, gameW, gameH/2 + 100, gameW, 100, foreground);
		Vector2 p0(gameW/2, gameH + 100);
		Vector2 p1(gameW/2, gameH/2 + 100);
		Vector2 p2(gameW/2+gameH/2 + 100, 0);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 19)
	{
		//al_draw_filled_rectangle(gameW/2 + 100, gameH/2 + 100, gameW, gameH, foreground);
		//al_draw_filled_triangle(gameW/2 + 100, gameH/2 + 100, gameW + 100, gameH/2 + 100, gameW + 100, 100, foreground);
		Vector2 p0(gameW/2 + 100, gameH + 100);
		Vector2 p1(gameW/2 + 100, gameH/2 + 100);
		Vector2 p2(gameW/2+gameH/2 + 200, 0);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	// sharp angle
	else if (whichImage == 20)
	{
		Vector2 wholeImageOffset(0, 0);
		if (whichImage == 20)
			wholeImageOffset.set(-100, 0);
		if (whichImage == 22)
			wholeImageOffset.set(100, 0);
		if (whichImage == 23)
			wholeImageOffset.set(-100, 100);
		if (whichImage == 24)
			wholeImageOffset.set(0, 100);
		if (whichImage == 25)
			wholeImageOffset.set(100, 100);
		wholeImageOffset *= zoom;

		Vector2 p0(0, 0);
		Vector2 p1(0, 1);
		Vector2 p2(1, 1);
		//Vector2 crop(gameW, gameH);
		Vector2 lineOrigin = crop/2 + wholeImageOffset;
		p0 = lineOrigin;
		p1.scaleToScreenEdge(lineOrigin, crop);
		p2.scaleToScreenEdge(lineOrigin, crop);
		p0 += pos - crop/2;
		p1 += pos - crop/2;
		p2 += pos - crop/2;

		al_draw_filled_triangle(p0.x, p0.y, p1.x, p1.y, p2.x, p2.y, foreground);

		// make the 45 degree line anti-aliased
		ALLEGRO_COLOR forehalf = al_map_rgb(foreground_r/2, foreground_g/2, foreground_b/2); // TODO: gamma correct
		al_lock_bitmap(al_get_target_bitmap(), ALLEGRO_PIXEL_FORMAT_ANY, 0);
		for (int x = p0.x; x < p2.x; x++)
			al_put_pixel(x, p0.y + (x-p0.x), forehalf);
		al_unlock_bitmap(al_get_target_bitmap());
	}
	else if (whichImage == 21)
	{
		//al_draw_filled_triangle(gameW/2, gameH/2, gameW/2, gameH, gameW, gameH, foreground);
		Vector2 p0(gameW/2, gameH);
		Vector2 p1(gameW/2, gameH/2);
		Vector2 p2(gameW, gameH);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 22)
	{
		//al_draw_filled_triangle(gameW/2 + 100, gameH/2, gameW/2 + 100, gameH, gameW + 100, gameH, foreground);
		Vector2 p0(gameW/2 + 100, gameH);
		Vector2 p1(gameW/2 + 100, gameH/2);
		Vector2 p2(gameW + 100, gameH);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 23)
	{
		//al_draw_filled_triangle(gameW/2 - 100, gameH/2 + 100, gameW/2 - 100, gameH + 100, gameW - 100, gameH + 100, foreground);
		Vector2 p0(gameW/2 - 100, gameH + 100);
		Vector2 p1(gameW/2 - 100, gameH/2 + 100);
		Vector2 p2(gameW - 100, gameH + 100);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 24)
	{
		//al_draw_filled_triangle(gameW/2, gameH/2 + 100, gameW/2, gameH + 100, gameW, gameH + 100, foreground);
		Vector2 p0(gameW/2, gameH + 100);
		Vector2 p1(gameW/2, gameH/2 + 100);
		Vector2 p2(gameW, gameH + 100);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
	else if (whichImage == 25)
	{
		//al_draw_filled_triangle(gameW/2 + 100, gameH/2 + 100, gameW/2 + 100, gameH + 100, gameW + 100, gameH + 100, foreground);
		Vector2 p0(gameW/2 + 100, gameH + 100);
		Vector2 p1(gameW/2 + 100, gameH/2 + 100);
		Vector2 p2(gameW + 100, gameH + 100);
		DrawRibbonAntiAliased(p0, p1, p2, foreground, 3);
	}
}
*/


void DrawTestImage(unsigned short whichImage, ALLEGRO_COLOR foregrColor, ALLEGRO_COLOR backgrColor, Vector2 pos, Vector2 crop, float zoom, ALLEGRO_BITMAP* bmpPrevTestImage)
{
	al_set_target_bitmap(bmpPrevTestImage);
	if (whichImage == 18)
		al_clear_to_color(foregrColor);
	else
	{
		al_clear_to_color(backgrColor);
		if (whichImage != 0)
		{
			al_lock_bitmap(bmpPrevTestImage, ALLEGRO_PIXEL_FORMAT_ANY, ALLEGRO_LOCK_READWRITE);
			//al_lock_bitmap(bmpPrevTestImage, ALLEGRO_PIXEL_FORMAT_ANY, ALLEGRO_LOCK_WRITEONLY); // works because DrawTestImage clears the bitmap to backgrColor color
			DrawTestImage(whichImage, foregrColor, backgrColor, pos, crop, zoom);
			al_unlock_bitmap(bmpPrevTestImage);
		}
	}
}

#endif
