#include <cmath>
#include <allegro5/allegro.h>
#include "Xiaoline.h"

using std::abs;
using std::fabs;

inline void swap(float& a, float& b)
{
	float tmp = a;
	a = b;
	b = tmp;
}

int iabs(int a)
{
	if (a < 0)
		return -a;
	return a;
}


namespace Emil
{

/*
template <typename TDrawingFunctor>
void Bresenline2(float x1, float y1, float x2, float y2)
{
	TDrawingFunctor drawFunc;

	bool steep = fabs(y2 - y1) > fabs(x2 - x1);
	if (steep)
	{
		swap(x1, y1);
		swap(x2, y2);
	}

	if (x1 > x2)
	{
		swap(x1, x2);
		swap(y1, y2);
	}

	int dx = (int)x2 - (int)x1;
	int dy = iabs((int)y2 - (int)y1);
	int error = -dx / 2;

	int ystep;
	if ((int)y1 < (int)y2)
		ystep = 1;
	else
		ystep = -1;

	int y = (int)y1;

	if (steep)
	{
		for (int x = (int)x1; x <= (int)x2; x++)
		{
			drawFunc((int)y, (int)x);
			error += dy;
			if (error > 0)
			{
				y += ystep;
				error -= dx;
			}
		}
	}
	else
	{
		for (int x = (int)x1; x <= (int)x2; x++)
		{
			drawFunc((int)x, (int)y);
			error += dy;
			if (error > 0)
			{
				y += ystep;
				error -= dx;
			}
		}
	}
}
*/

void Bresenline(float x1, float y1, float x2, float y2, void(*drawFunc)(int x, int y, const bool horizontal, float fraction, void* pExtraArgs), void* pExtraArgs)
{
	bool steep = fabs(y2 - y1) > fabs(x2 - x1);
	if (steep)
	{
		swap(x1, y1);
		swap(x2, y2);
	}

	if (x1 > x2)
	{
		swap(x1, x2);
		swap(y1, y2);
	}

	int dx = (int)x2 - (int)x1;
	int dy = iabs((int)y2 - (int)y1);
	int error = -dx / 2;

	int ystep;
	if ((int)y1 < (int)y2)
		ystep = 1;
	else
		ystep = -1;

	int y = (int)y1;

	if (steep)
	{
		for (int x = (int)x1; x <= (int)x2; x++)
		{
			//drawFunc((int)y, (int)x, pExtraArgs);
			// no, i'll use the same callback function for both bresenham and xiaowu! easier to switch between :)
			drawFunc((int)y, (int)x, true, 0, pExtraArgs);

			error += dy;
			if (error > 0)
			{
				y += ystep;
				error -= dx;
			}
		}
	}
	else
	{
		for (int x = (int)x1; x <= (int)x2; x++)
		{
			//drawFunc((int)x, (int)y, pExtraArgs);
			// no, i'll use the same callback function for both bresenham and xiaowu! easier to switch between :)
			drawFunc((int)x, (int)y, false, 0, pExtraArgs);

			error += dy;
			if (error > 0)
			{
				y += ystep;
				error -= dx;
			}
		}
	}
}


//void Xiaoline(float x1, float y1, float x2, float y2, float r, float g, float b)
void Xiaoline(float x1, float y1, float x2, float y2, void(*drawFunc)(int x, int y, const bool horizontal, float fraction, void* pExtraArgs), void* pExtraArgs)
{
//	cout << "Xiaoline(" << x1 << ", " << y1 << ", " << x2 << ", " << y2 << ")" << endl;

	bool steep = fabs(y2 - y1) > fabs(x2 - x1);
	if (steep)
	{
		swap(x1, y1);
		swap(x2, y2);
	}

	if (x1 > x2)
	{
		swap(x1, x2);
		swap(y1, y2);
	}

	float dx = x2 - x1;
	float dy = y2 - y1;
	float intery = dy / dx;


	float y = y1;
/*
	for (float x = x1; x <= x2; x++)
	{
		float fractiony = y - (int)y;

		if (steep)
			drawFunc((int)y, (int)x, steep, fractiony, pCallbackArgs);
		else
			drawFunc((int)x, (int)y, steep, fractiony, pCallbackArgs);

		y += intery;
	}
*/
	// optimization: put the if() outside the loop
	if (steep)
	{
		for (float x = x1; x < x2; x++)
		{
			drawFunc((int)y, (int)x, true, y - (int)y, pExtraArgs);
			y += intery;
		}
	}
	else
	{
		for (float x = x1; x < x2; x++)
		{
			drawFunc((int)x, (int)y, false, y - (int)y, pExtraArgs);
			y += intery;
		}
	}
}



/* CALLBACK FUNCTIONS: ***********************************************************/


/* example of a simple callback function which can only draw white lines on black background:

void XiaolineDrawGrey(int x, int y, const bool horizontal, float fraction, void* pObject)
{
	// horizontal=true means we should draw (x;y) and (x+1;y)
	// horizontal=false means we should draw (x;y) and (x;y+1)

	// let's pretend we have a fixed amount of ink
	// fraction=0 means all the ink should go in the first pixel
	// fraction=0.5 means they should divide the ink equally
	// fraction=1 means all the ink should go in the second pixel

	fraction = 1.0f - fraction;
	al_put_pixel(x, y, al_map_rgb_f(fraction,fraction,fraction));

	if (horizontal)
		x++;
	else
		y++;

	fraction = 1.0f - fraction;
	al_put_pixel(x, y, al_map_rgb_f(fraction,fraction,fraction));
}
*/


void GrayDrawFunc(int x, int y, const bool horizontal, float fraction, void* pObject) // used for callback by Xiaoline
{
	// draws two pixels
	// horizontal=true means we should draw (x;y) and (x+1;y)
	// horizontal=false means we should draw (x;y) and (x;y+1)

	// let's pretend we have a fixed amount of ink
	// fraction=0 means all the ink should go in the first pixel
	// fraction=0.5 means they should divide the ink equally
	// fraction=1 means all the ink should go in the second pixel

	// and the amount of ink we have is given by 'alpha'
	float alpha = *((float*)pObject);

	al_put_blended_pixel(x, y, al_map_rgba_f(0.5f,0.5f,1.0f, alpha*(1.0f - fraction)));

	if (horizontal)
		x++;
	else
		y++;

	al_put_blended_pixel(x, y, al_map_rgba_f(0.5f,0.5f,1.0f, alpha*fraction));
}



void AlphaLine::ReallyDraw(int x, int y, const bool horizontal, float fraction)
{
	// draws two pixels
	// horizontal=true means we should draw (x;y) and (x+1;y)
	// horizontal=false means we should draw (x;y) and (x;y+1)

	// let's pretend we have a fixed amount of ink
	// fraction=0 means all the ink should go in the first pixel
	// fraction=0.5 means they should divide the ink equally
	// fraction=1 means all the ink should go in the second pixel

	// and the amount of ink we have is given by 'alpha'

	al_put_blended_pixel(x, y, al_map_rgba_f(r,g,b, alpha*(1.0f - fraction)));

	if (horizontal)
		x++;
	else
		y++;

	al_put_blended_pixel(x, y, al_map_rgba_f(r,g,b, alpha*fraction));
}




}
