#include <stdio.h>
#include <cstring>
#include <fstream>

#include "PenFrame.h"

using namespace Emil;
using std::ios;
using std::ifstream;
using std::ofstream;

const int inputGameW = 1280; const int inputGameH = 1024;

bool ReadTriplogFromFileIntoMemory(char* const& infilename, PenFrame_v2*& penFrame, unsigned int& nFrames)
{
	ifstream fin;
	printf("infilename=%s\n", infilename);
	fin.open(infilename, ios::in | ios::binary);
	if (fin.fail())
	{
		printf("fin.open failed\n");
		return false;
	}

	fin.seekg(0, ios::end);
	unsigned int nBytes = fin.tellg();
	printf("nBytes=%u\n", nBytes);
	fin.seekg(0, ios::beg);

	// read the file stream into the penFrame[] array

	nFrames = nBytes / sizeof(PenFrame_v2);
	printf("sizeof(PenFrame_v2)=%u\n", (unsigned int)sizeof(PenFrame_v2));
	printf("nFrames=%u\n", nFrames);
	printf("nFrames * sizeof(PenFrame_v2)=%u\n", nFrames * sizeof(PenFrame_v2));
	if (nFrames == 0)
		return false;
	penFrame = new PenFrame_v2[nFrames];
	if (!penFrame)
	{
		printf("new PenFrame_v2[%u]; failed\n", nFrames);
		return false;
	}

	for (unsigned long i = 0; i < nFrames; i++)
	{
		fin.read((char*)&(penFrame[i]), sizeof(PenFrame_v2));
		if (!fin.good())
		{
			printf("!fin.good()\n");
			fin.close();
			return false;
		}
	}
	// TODO: try this instead, shorter and faster?
	//fin.read((char*)&(penFrame[0]), sizeof(PenFrame_v2)*nFrames);

	fin.close();
	return true;
}


void JoinFiles()
{
	// paste all the consequitive .oevlog files together
	// with some artificial time gaps in between

	const int numberOfInFiles = 7;
	char infilenames[numberOfInFiles][40] =
	{
		"timedrawing_2013-11-30_18.35.59.oevlog",
		"timedrawing_2013-11-30_19.01.59.oevlog",
		"timedrawing_2013-11-30_21.19.59.oevlog",
		"timedrawing_2013-11-30_21.43.59.oevlog",
		"timedrawing_2014-04-26_14.27.59.oevlog",
		"timedrawing_2014-04-26_16.57.59.oevlog",
		"timedrawing_2014-04-26_19.01.59.oevlog"
	};

	char outfilename[] = "timedrawing_2013-11-30.oevlog ";


	PenFrame_v2** penFrame = new PenFrame_v2*[numberOfInFiles];
	unsigned int* nFrames = new unsigned int[numberOfInFiles];

	for (int iInFile = 0; iInFile < numberOfInFiles; iInFile++)
	{
		ReadTriplogFromFileIntoMemory(infilenames[iInFile], penFrame[iInFile], nFrames[iInFile]);
	}


	ofstream fout;
	fout.open(outfilename, ios::out | ios::trunc | ios::binary);
	printf("outfilename=%s\n", outfilename);


	// find long boring time gaps in the files, and shrink them
	for (int iInFile = 0; iInFile < numberOfInFiles; iInFile++)
	{
		for (unsigned long i = 1; i < nFrames[iInFile]; i++)
		{
			long gapLength = penFrame[iInFile][i].wacomtime - penFrame[iInFile][i-1].wacomtime;
			const long acceptedGapLength = 1000 * 30;
			if (gapLength > acceptedGapLength)
			{
				// move this frame, and all subsequent frames, back so the gap gets shorter
				long adjust = gapLength - acceptedGapLength;
				for (unsigned long j = i; j < nFrames[iInFile]; j++)
				{
					penFrame[iInFile][j].wacomtime -= adjust;
				}
			}
		}
	}


	int globalWacomtimeOffset = 0;
	int globalFrameIndexOffset = 0;

	for (int iInFile = 0; iInFile < numberOfInFiles; iInFile++)
	{
		if (iInFile)
		{
			// if one file ends with test image X, and the next file also begins with test image X
			if (penFrame[iInFile-1][nFrames[iInFile-1]-1].testimage == penFrame[iInFile][0].testimage)
			{
				// add 30 sec
				const int secGap = 30;
				globalWacomtimeOffset += 1000 * secGap;
				// how many frames is that? the tablet sends roughly 133,33 frames per sec, or 400 frames per 3 sec
				globalFrameIndexOffset += 400 * secGap / 3;
			}
			else
			{
				// add 10 sec, then switch testimage, then 20 sec more
				const int secGapBefore = 10;
				globalWacomtimeOffset += 1000 * secGapBefore;
				globalFrameIndexOffset += 400 * secGapBefore / 3;

				PenFrame_v2 f;
				f.wacomtime = globalWacomtimeOffset;
				f.index = globalFrameIndexOffset;
				f.penpressure = 0;
				f.testimage = penFrame[iInFile][0].testimage;
				f.penx = 0;
				f.peny = 0;
				fout.write((char*)(&f), sizeof(PenFrame_v2));
				const int secGapAfter = 20;
				globalWacomtimeOffset += 1000 * secGapAfter;
				globalFrameIndexOffset += 400 * secGapAfter / 3;
			}
		}

		unsigned long firstWacomTimeInThisFile = penFrame[iInFile][0].wacomtime;
		unsigned int firstIndexInThisFile = penFrame[iInFile][0].index;

		for (unsigned long i = 0; i < nFrames[iInFile]; i++)
		{
			penFrame[iInFile][i].wacomtime += globalWacomtimeOffset - firstWacomTimeInThisFile;
			penFrame[iInFile][i].index += globalFrameIndexOffset - firstIndexInThisFile;
			//penFrame[iInFile][i].penx -= inputGameW/2;
			//penFrame[iInFile][i].peny -= inputGameH/2;

			fout.write((char*)&(penFrame[iInFile][i]), sizeof(PenFrame_v2));
		}

		globalWacomtimeOffset = penFrame[iInFile][nFrames[iInFile]-1].wacomtime;
		globalFrameIndexOffset = penFrame[iInFile][nFrames[iInFile]-1].index;

		if (!fout.good())
		{
			printf("!fout.good()\n");
			break;
		}
	}

	printf("done\n");
	fout.close();

	for (int iInFile = 0; iInFile < numberOfInFiles; iInFile++)
	{
		delete[] penFrame[iInFile];
		penFrame[iInFile] = 0;
	}
	delete[] nFrames;
	delete[] penFrame;
}

void ConvertFromBinaryToCsv()
{
	PenFrame_v2* penFrame = 0;
	unsigned int nFrames = 0;

	char infilename[] = "timedrawing_2013-11-30.oevlog"; // TODO: get it from argv instead

	ReadTriplogFromFileIntoMemory(infilename, penFrame, nFrames);

	char outfilename[strlen(infilename)+5];
	sprintf(outfilename, "%s.csv", infilename);
	printf("outfilename=%s\n", outfilename);

	ofstream fout;
	fout.open(outfilename, ios::out | ios::trunc);

	for (unsigned long i = 0; i < nFrames; i++)
	{
		fout << penFrame[i].wacomtime;
		fout << ',';
		fout << penFrame[i].index;
		fout << ',';
		fout << penFrame[i].penpressure;
		fout << ',';
		fout << penFrame[i].testimage;
		fout << ',';
		fout << penFrame[i].penx;
		fout << ',';
		fout << penFrame[i].peny;
		//fout.put('\r'); // RFC4180 specifies dos&windows style newlines, but I think it's a waste of space
		fout.put('\n');
	}

	if (!fout.good())
	{
		printf("!fout.good()\n");
	}

	delete[] penFrame;
}

int main()
{
	//JoinFiles();
	//ConvertFromBinaryToCsv();

	return 0;
}
