/*
Copyright (c) 2014 hallucination-research.com

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/
#include <allegro5/allegro.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_windows.h>

#include <stdio.h>
#include <windows.h>
#include <cmath>
#include <ctime>
#include <fstream>

#include <wintab.h>
#define PACKETDATA	(PK_X | PK_Y | PK_TIME | PK_NORMAL_PRESSURE)
#define PACKETMODE	0
#include <pktdef.h>
#include "wintab_utils.h"

#include "Vector2.h"
#include "Xiaoline.h"
#include "PenFrame.h"


using namespace Emil;
using std::ios;
using std::ofstream;

const int FRAMES_PER_SECOND = 60;
const double SECONDS_BETWEEN_UPDATES = 1.0f/FRAMES_PER_SECOND;
const int gameW = 1280; const int gameH = 1024;
//const int gameW = 1440; const int gameH = 900;

bool plzDeactivateSecMonitor = false;

int InitWacomTablet(ALLEGRO_DISPLAY* display, HCTX& hTab, LONG& maxPressureLevel, LONG& minPenX, LONG& minPenY, LONG& maxPenX, LONG& maxPenY)
{
	if ( !LoadWintab( ) )
	{
		printf("Wintab32.dll not available\n");
		return 0;
	}

	if (!gpWTInfoA(0, 0, NULL))
	{
		printf("Tablet drivers are not installed (or no tablet is attached?)\n");
		return 0;
	}


	int numDevices;
	gpWTInfoA(WTI_INTERFACE, IFC_NDEVICES, &numDevices);
	if (numDevices < 1)
	{
		printf("No tablet is currently attached to your computer\n");
		return 0;
	}


	AXIS asdf;
	gpWTInfoA(WTI_DEVICES, DVC_NPRESSURE, &asdf);
	maxPressureLevel = asdf.axMax;

	LOGCONTEXT lcMine;
	//gpWTInfoA(WTI_DEFCONTEXT, 0, &lcMine);
	gpWTInfoA(WTI_DEFSYSCTX, 0, &lcMine);

	// The damn tablet driver won't give me "raw" position data from its sensors.
	// It seems to convert them to windows virtual desktop coordinates. I found no way around it :(
	// But it's only a problem if the desktop is larger than the screen.
	if (lcMine.lcSysOrgX < 0 || lcMine.lcSysOrgY < 0)
	{
		plzDeactivateSecMonitor = true;
		printf("Plz deactivate your secondary monitor before!\n");
		MessageBeep(MB_ICONEXCLAMATION);
	}

	lcMine.lcPktData = PACKETDATA;
	lcMine.lcPktMode = PACKETMODE;
	lcMine.lcMoveMask = PACKETDATA;

	lcMine.lcOutOrgX = 0;
	lcMine.lcOutOrgY = 0;
	//lcMine.lcOutExtX = 5103;
	//lcMine.lcOutExtY = 3711;

	lcMine.lcOutExtX = gameW;
	lcMine.lcOutExtY = gameH;
	minPenX = 0;
	minPenY = 0;
	maxPenX = lcMine.lcOutExtX - 1;
	maxPenY = lcMine.lcOutExtY - 1;

/*
	// with this, i get 2 <= x <= 10239 and 2 <= y <= 8189
	lcMine.lcOutExtX = 10240; //gameW * 8;
	lcMine.lcOutExtY = 8192; //gameH * 8;
	minPenX = 2;
	minPenY = 2;
	maxPenX = 10239;
	maxPenY = 8189;
*/

	//lcMine.lcOptions = (lcMine.lcOptions ^ CXO_MARGIN) | CXO_MGNINSIDE;
	lcMine.lcOptions = CXO_SYSTEM;

	// open the region
	HWND hWnd = al_get_win_window_handle(display);
	hTab = gpWTOpenA(hWnd, &lcMine, TRUE);
	if (!hTab)
	{
		printf("Could Not Open Tablet Context.\n");
		return 0;
	}

	return 1;
}



static float volume = 1.f;
static bool playDrawingSound = false;

// humm, for this program, it's stupid to fill the buffers over and over again with the same things
// TODO: init two buffers at program start, one with silence, one with noise. then just feed them into waveOutWrite over and over again

void mySoundProc(void *pSoundBuffer, long bufferLen)
{
	// Convert params, assuming we create a 16bits, mono waveform.
	signed short *pSample = (signed short*)pSoundBuffer;
	long nbSample = bufferLen / sizeof(signed short);
	//printf("nbSample=%d\n", nbSample); // nbSample=2205

	if (playDrawingSound)
	{
		// noise
		signed short prev = rand();
		for (int i = 0; i < nbSample; i++)
		{
			signed short current = rand();
			signed short noise = (current - prev) / 4; prev = current; // violet/purple noise
			//signed short noise = current / 2; // white noise
			// noise is now between -16384 and 16383
			*pSample = (signed short)(volume * noise);
			pSample++;
		}
	}
	else
	{
		for (int i = 0; i < nbSample; i++)
		{
			*pSample = 0;
			pSample++;
		}
	}
}

void FillSoundBufferWithNoise(void *pSoundBuffer, long bufferLen, unsigned short vol)
{
	// vol is a fixed-point number
	// it will be divided by 256 when used

	// Convert params, assuming we create a 16bits, mono waveform.
	signed short *pSample = (signed short*)pSoundBuffer;
	long nbSample = bufferLen / sizeof(signed short);
	//printf("nbSample=%d\n", nbSample); // nbSample=2205

	if (vol == 0)
	{
		for (int i = 0; i < nbSample; i++)
		{
			*pSample = 0;
			pSample++;
		}
	}
	else
	{
		// noise
		signed short prev = rand();
		for (int i = 0; i < nbSample; i++)
		{
			signed short current = rand();
			signed short noise = (current - prev) / 4; prev = current; // violet/purple noise
			//signed short noise = current / 2; // white noise
			// noise is now between -16384 and 16383
			*pSample = (signed short)(noise * vol / 256);
			pSample++;
		}
	}
}


#include "DrawTestImage.h"

/*
void DrawTestImage(unsigned short currentTestImage, ALLEGRO_COLOR foreground, ALLEGRO_COLOR background)
{
	float zoom = 1;
	Vector2 pos(gameW/2,gameH/2); // TODO: same as eyeTargetX and Y
	Vector2 crop(gameW,gameH);
	DrawTestImage(currentTestImage, foreground, background, pos, crop, zoom, al_get_target_bitmap());
}
*/

void LoadTestImageFromDisk(unsigned short currentTestImage, char* testImageDir)
{
	char* fullFilePath = new char[strlen(testImageDir) + strlen("testimage_00.bmp") + 3];
	sprintf(fullFilePath, "%stestimage_%02d.bmp", testImageDir, currentTestImage);
	ALLEGRO_BITMAP* testimage = al_load_bitmap(fullFilePath);
	delete[] fullFilePath;
	if (testimage)
	{
		al_draw_bitmap(testimage, 0, 0, 0);
		al_destroy_bitmap(testimage);
	}
}

int main(int argc, char* argv[])
{
	// Init the Allegro graphics&input library

	if (!al_init())
	{
		printf("al_init() failed\n");
		return 1;
	}

	if (!al_install_keyboard())
	{
		printf("al_install_keyboard() failed\n");
		return 1;
	}

	//al_set_new_display_flags(ALLEGRO_FULLSCREEN);
	al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW);

	ALLEGRO_DISPLAY* display = al_create_display(gameW, gameH);
	if (!display)
	{
		printf("al_create_display(%d,%d) failed\n", gameW, gameH);
		return 1;
	}

	ALLEGRO_EVENT_QUEUE* event_queue = al_create_event_queue();
	if (!event_queue)
	{
		printf("al_create_event_queue() failed\n");
		return 1;
	}
	al_register_event_source(event_queue, al_get_display_event_source(display));
	ALLEGRO_EVENT al_event;

	if (!al_init_primitives_addon())
	{
		printf("al_init_primitives_addon() failed\n");
		return 1;
	}

	if (!al_init_image_addon())
	{
		printf("al_init_image_addon() failed\n");
		return 1;
	}

	al_init_font_addon();
	// Note that if you intend to load bitmap fonts, you will need to initialise allegro_image separately
	// (unless you are using another library to load images

    ALLEGRO_FONT *f1 = al_load_font("a4_font.tga", 16, 0);
    if (!f1)
	{
		printf("al_load_font() failed\n");
		return 1;
	}


	ALLEGRO_COLOR foreground;
	{
		const int foreground_r = 146/2;
		const int foreground_g = 98/2;
		const int foreground_b = 255/2;
		foreground = al_map_rgb(foreground_r, foreground_g, foreground_b);
	}
	ALLEGRO_COLOR background = al_map_rgb(36, 24, 63);
	al_clear_to_color(background);

	srand((int)(al_get_time()*10000));


	ALLEGRO_BITMAP* bmpcursor;
	bmpcursor = al_load_bitmap("cursor.png");
	ALLEGRO_MOUSE_CURSOR* custom_cursor;
	custom_cursor = 0;

	if (!al_install_mouse())
	{
		printf("al_install_mouse() failed\n");
		return 1;
	}

	custom_cursor = al_create_mouse_cursor(bmpcursor, 15, 15);
	if (!custom_cursor)
	{
		printf("Error creating mouse cursor\n");
		return 1;
	}
	al_destroy_bitmap(bmpcursor);
	bmpcursor = NULL;

	al_set_mouse_cursor(display, custom_cursor);

	al_clear_to_color(background);




	// Init Wacom tablet

	HCTX hTab = 0;
	LONG maxPressureLevel = 0;
	LONG minPenX = 0;
	LONG minPenY = 0;
	LONG maxPenX = 0;
	LONG maxPenY = 0;
	if (InitWacomTablet(display, hTab, maxPressureLevel, minPenX, minPenY, maxPenX, maxPenY))
	{
		printf("Using tablet.\n");
		printf("maxPenX=%d.\n", (int)maxPenX);
		printf("maxPenY=%d.\n", (int)maxPenY);
	}
	else
	{
		printf("Falling back to mouse instead.\n");
		MessageBeep(MB_ICONEXCLAMATION);
		maxPressureLevel = 1023;
		minPenX = 0;
		minPenY = 0;
		maxPenX = gameW - 1;
		maxPenY = gameH - 1;
	}



	//const float eyeTargetX = gameW / 2;
	//const float eyeTargetY = gameH / 2;

	unsigned short imageSequence[] = {
		1,0,
		2,0,
		3,0,
		4,0,
		5,0,
		6,0,
		7,0,
		8,0,
		9,0,
		10,0,
		11,0,
		12,0,
		13,0,
		14,0,
		15,0,
		16,0,
		17,0,
		20,0,
		21,0,
		22,0,
		23,0,
		24,0,
		19,0,
		47,0,
		23,0,
		24,0,
		25,
		26,0,
		27,
		28,18,
		29,0,
		30,0,
		31,0,
		32,0,
		33,0,
		34,0,
		35,18,
		36,0,
		37,18,
		38,0,
		39,0,
		40,0,
		41,0,
		42,18,
		43,0,
		44,18,
		45,
		46
	};
	/* TODO: try more switching without a blank image in between, the transition period is interesting
		3,// from small circle
		7,// to big circle
		9,// to line
		15,// to blunt angle
		12,// to straight angle
		21,// to sharp angle
		12,// back to straight angle
		15,// to blunt
		9,// to line
		7,// to big circle
		3// to small circle
	*/
	const unsigned short sequenceLength = sizeof(imageSequence) / sizeof(*imageSequence);
	unsigned short currentSequencePos = 0;
	unsigned short currentTestImage = imageSequence[currentSequencePos];

	currentTestImage = imageSequence[currentSequencePos];
	//DrawTestImage(currentTestImage, foreground, background);
	//al_put_pixel(eyeTargetX, eyeTargetY, al_map_rgb(127,127,127));
	//al_put_pixel(eyeTargetX-1, eyeTargetY, al_map_rgb(0,0,0));
	//al_put_pixel(eyeTargetX, eyeTargetY-1, al_map_rgb(0,0,0));
	//al_put_pixel(eyeTargetX-1, eyeTargetY-1, al_map_rgb(127,127,127));
	char testImageDir[] = "testimages/"; // relative, only works from the right working dir
	//char testImageDir[] = "e:/gamedev/inwork/open_eye_visuals/timedrawing/testimages/"; // absolute, always works on my computer
	LoadTestImageFromDisk(currentTestImage, testImageDir);
	al_flip_display();


	char filename[] = "timedrawing_2011-12-31_23_59_59.oevlog                         ";
	ofstream fout;
	{
		time_t t = time(NULL);
		struct tm* timeinfo = localtime(&t);
		char datestring[] = "2011-12-31_23.59.59 ";
		strftime(datestring, 20, "%Y-%m-%d_%H.%M.%S", timeinfo);
		sprintf(filename, "timedrawing_%s.oevlog", datestring);
		printf("writing to filename=%s\n", filename);
		fout.open(filename, ios::out | ios::binary);
		if (fout.fail())
		{
			printf("fout.open failed\n");
			return 1;
		}
	}



	// -1=just released
	//  0=not held down
	//  1=just pressed for the first time
	//  2=held down
	int mouse1state = 0;
	int mouse2state = 0;
	int spaceButtonState = 0;

	Vector2 prevmouse;
	ALLEGRO_MOUSE_STATE mousestate;
	al_get_mouse_state(&mousestate);

	ALLEGRO_KEYBOARD_STATE key_state;
	int gameRunning = 1;
	unsigned long frameIndex = 0;
	unsigned long nFramesWithoutPressure = 0;
	unsigned long nFramesThisImageHasBeenShown = 0;
	double timePrev = al_get_time();

	//unsigned long thisSecond = (unsigned long)timePrev;
	//int fps = 0;
	//int nFramesThisSecond = 0;

	while (gameRunning)
	{
		// simple timing: lock the main loop to a fixed speed
		// slower computers will get slowdowns, faster computers will al_rest to use less cpu
		// but al_rest has quite bad accuracy, so this code shouldn't be used in any serious project
		double timeNow = al_get_time();
		//printf("timeNow=%f\n", timeNow);
		double deltaTime = timeNow - timePrev;
		//printf("deltaTime=%f\n", deltaTime);
		double sleeptime = SECONDS_BETWEEN_UPDATES - deltaTime;
		if (sleeptime > 0)
		{
			al_rest(sleeptime);
		}
		//printf("sleep error=%f\n", ((timeNow + sleeptime) - al_get_time()));
		timePrev = timeNow + sleeptime;
		//timeNow = al_get_time();


		if (al_get_next_event(event_queue, &al_event))
		{
			if (al_event.type == ALLEGRO_EVENT_DISPLAY_CLOSE)
				gameRunning = 0;
		}

		al_get_keyboard_state(&key_state);
		if (al_key_down(&key_state, ALLEGRO_KEY_ESCAPE))
		{
			gameRunning = 0;
		}


		prevmouse.x = mousestate.x;
		prevmouse.y = mousestate.y;

		al_get_mouse_state(&mousestate);
		if (mousestate.buttons & 1)
		{
			if (mouse1state < 1)
				mouse1state = 1;
			else
				mouse1state = 2;
		}
		else
		{
			if (mouse1state > 0)
				mouse1state = -1;
			else
				mouse1state = 0;
		}

		if (mousestate.buttons & 2)
		{
			if (mouse2state < 1)
				mouse2state = 1;
			else
				mouse2state = 2;
		}
		else
		{
			if (mouse2state > 0)
				mouse2state = -1;
			else
				mouse2state = 0;
		}

		if (al_key_down(&key_state, ALLEGRO_KEY_SPACE))
		{
			if (spaceButtonState < 1)
				spaceButtonState = 1;
			else
				spaceButtonState = 2;
		}
		else
		{
			if (spaceButtonState > 0)
				spaceButtonState = -1;
			else
				spaceButtonState = 0;
		}



		PenFrame_v2 f;
		f.testimage = currentTestImage;

		bool anyPressure = false;
		if (hTab)
		{
			// our Wacom tablet sends roughly 133,33 packets per sec, or 400 packets per 3 sec
			const int cMaxPkts = 10;
			PACKET pkts[cMaxPkts];
			int nPacketsReturned = gpWTPacketsGet(hTab, cMaxPkts, &pkts);
			if (nPacketsReturned)
			{
				for (int i = 0; i < nPacketsReturned; i++)
				{
					PACKET pkt = pkts[i];
					f.wacomtime = pkt.pkTime;
					f.index = frameIndex;
					f.penx = pkt.pkX;
					f.peny = (maxPenY - pkt.pkY) + minPenY; // flip vertical axis
					f.penpressure = pkt.pkNormalPressure;
					frameIndex++;
					if (f.penpressure > 0)
					{
						anyPressure = true;
						// TODO: handle little-endian or big-endian
						fout.write((char*)(&f), sizeof(PenFrame_v2));
					}
				}
			}
		}
		else
		{
			f.wacomtime = GetTickCount();
			f.index = frameIndex;
			f.penx = mousestate.x;
			f.peny = mousestate.y;
			if (mouse1state > 0)
				f.penpressure = 1023; // normal computer mice don't report pressure
			frameIndex++;
			if (f.penpressure > 0)
			{
				anyPressure = true;
				// TODO: handle little-endian or big-endian
				fout.write((char*)(&f), sizeof(PenFrame_v2));
			}
		}

		// make a backup of the output file when the user seems idle
		// (not when he is active, because it would make the interface feel unresponsive)
		// backup to a different hard disk
		if (anyPressure)
			nFramesWithoutPressure = 0;
		else
			nFramesWithoutPressure++;
		if (nFramesWithoutPressure == FRAMES_PER_SECOND*10) // 10 seconds
		{
			char backup[] = "d:\\timedrawing_2011-12-31_23_59_59.oevlog                         ";
			sprintf(backup, "d:\\%s", filename);
			CopyFile(filename, backup, FALSE);
		}
		if (nFramesWithoutPressure < FRAMES_PER_SECOND*10) // 10 seconds
		{
			nFramesThisImageHasBeenShown++;
			unsigned long nFramesThisImageShouldBeShown = 10 * 60;
			if (currentTestImage == 0 || currentTestImage == 18)
				nFramesThisImageShouldBeShown = 40;
			nFramesThisImageShouldBeShown *= FRAMES_PER_SECOND;
			if (nFramesThisImageHasBeenShown == nFramesThisImageShouldBeShown)
			{
				MessageBeep(MB_ICONEXCLAMATION); // suggest to the user that it's time to switch image
				nFramesThisImageHasBeenShown *= 0.8; // and remind him again in 2 minutes
			}
		}


		if (f.penpressure > 0)
		{
			volume = (float)f.penpressure/maxPressureLevel;
			//volume = (float)f.penx/maxPenX;
			playDrawingSound = true;
		}
		else
		{
			playDrawingSound = false;
		}






		// graphics

		bool graphicsChanged = false;

		// void al_set_target_backbuffer(ALLEGRO_DISPLAY *display)
		// is the same as al_set_target_bitmap(al_get_backbuffer(display));

		if (spaceButtonState == 1)
		{
			currentSequencePos = (currentSequencePos + 1) % sequenceLength;
			currentTestImage = imageSequence[currentSequencePos];
			//DrawTestImage(currentTestImage, foreground, background);
			LoadTestImageFromDisk(currentTestImage, testImageDir);
			graphicsChanged = true;
			nFramesThisImageHasBeenShown = 0;
		}

		/*
		nFramesThisSecond++;
		if (thisSecond != (unsigned long)timePrev)
		{
			thisSecond = (unsigned long)timePrev;
			fps = nFramesThisSecond;
			nFramesThisSecond = 0;
		}

		al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 10,10, 0, "%d", fps);

		al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,32, 0, "%d", (int)frameIndex);
		*/

		if (graphicsChanged)
		{
			//al_put_pixel(eyeTargetX, eyeTargetY, al_map_rgb(127,127,127));
			//al_put_pixel(eyeTargetX-1, eyeTargetY, al_map_rgb(0,0,0));
			//al_put_pixel(eyeTargetX, eyeTargetY-1, al_map_rgb(0,0,0));
			//al_put_pixel(eyeTargetX-1, eyeTargetY-1, al_map_rgb(127,127,127));

			//al_draw_textf(f1, al_map_rgb(64, 96, 64), 0,0, 0, "%02u", currentTestImage);

			al_flip_display();
		}
	}

	fout.close();

	al_destroy_mouse_cursor(custom_cursor);
	al_destroy_display(display);

	return 0;
}


