/*
This program just outputs all the test images as pcx files
*/

#include <allegro5/allegro.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_font.h>

#include <stdio.h>
#include <windows.h>
#include <cmath>
#include <ctime>
#include <fstream>

#include "debuglog.h"
#include "Vector2.h"
#include "Polar2.h"
#include "Xiaoline.h"
#include "Circle.h"
#include "Polygon.h"
#include "Ribbon.h"
#include "misc.h"


using namespace Emil;

const int gameW = 1280; const int gameH = 1024;

#include "DrawTestImage.h"

//# define WOFF(x,y) ((y) * gameW + (x))
inline int WOFF(int x, int y)
{
	return y * gameW + x;
}


int main(int argc, char* argv[])
{
	if (!al_init())
	{
		abort_with_message("al_init() failed\n");
		return 1;
	}

	//al_set_new_display_flags(ALLEGRO_FULLSCREEN);
	//al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW | ALLEGRO_SINGLE_BUFFER);
	al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW);

	ALLEGRO_DISPLAY* display = al_create_display(gameW, gameH);
	if (!display)
	{
		abort_with_message("al_create_display(%d,%d) failed\n", gameW, gameH);
		return 1;
	}

	if (!al_init_primitives_addon())
	{
		abort_with_message("al_init_primitives_addon() failed\n");
		return 1;
	}

	if (!al_init_image_addon())
	{
		abort_with_message("al_init_image_addon() failed\n");
		return 1;
	}

	al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP);


	ALLEGRO_COLOR foregrColor;
	{
		const int foreground_r = 73;
		const int foreground_g = 49;
		const int foreground_b = 127;
		foregrColor = al_map_rgb(foreground_r, foreground_g, foreground_b);
	}
	ALLEGRO_COLOR backgrColor = al_map_rgb(36, 24, 63);


	ALLEGRO_BITMAP* bmpTestImage = al_create_bitmap(gameW, gameH);
	if (!bmpTestImage)
	{
		abort_with_message("bmpTestImage = al_create_bitmap() failed\n");
		return 1;
	}

	Vector2 pos(gameW/2,gameH/2);
	Vector2 crop(gameW,gameH);

	for (int i = 23; i <= 60; i++)
	{
		al_set_target_bitmap(bmpTestImage);
		al_clear_to_color(backgrColor);
		al_lock_bitmap(bmpTestImage, ALLEGRO_PIXEL_FORMAT_ANY, ALLEGRO_LOCK_READWRITE);

		DrawTestImage(i, foregrColor, backgrColor, pos, crop, 1);

		al_put_pixel(pos.x, pos.y, al_map_rgb(127,127,127));
		al_put_pixel(pos.x-1, pos.y, al_map_rgb(0,0,0));
		al_put_pixel(pos.x, pos.y-1, al_map_rgb(0,0,0));
		al_put_pixel(pos.x-1, pos.y-1, al_map_rgb(127,127,127));

		al_unlock_bitmap(bmpTestImage);

		char pcxFileName[] = "testimage_01.bmp";
		sprintf(pcxFileName, "testimage_%02d.bmp", i);
		al_save_bitmap(pcxFileName, bmpTestImage);
	}

	al_destroy_display(display);

	return 0;
}
