// All of these functions are small, and we typically want them to execute quickly.
// Therefore, they are all defined in this header file, instead of a separate misc.cpp file.
// Also, they are all given the "inline" keyword.
// Using a cpp would make inlining impossible.

#ifndef MISC_H_INCLUDED
#define MISC_H_INCLUDED

namespace Emil
{

// When t = 0 returns from.
// When t = 1 returns to.
// When t = 0.5 returns the point midway between from and to.
inline float Lerp(float from, float to, float t)
{
	return (from * (1-t)) + (to * t);
}

// When value = from, returns 0
// When value = to, returns 1
// When value is midway between from and to, returns 0.5
// When value is outside the range, returns <0 or >1 as you would expect
// Be careful, if to==from you get division by 0 and an undefined result
inline float InverseLerp(float from, float to, float value)
{
	return (value - from) / (to - from);
}

// Like InverseLerp but avoids divison by 0
inline float InverseLerpZerosafe(float from, float to, float value)
{
	if (from == to)
		return 0.5;
	return (value - from) / (to - from);
}

// Like InverseLerp but never returns a value outside of [0,1]
inline float InverseLerpClamped(float from, float to, float value)
{
	value = InverseLerp(from, to, value);
	if (value < 0) return 0;
	if (value > 1) return 1;
	return value;
}

}

#endif
