/*
Copyright (c) 2014 hallucination-research.com

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

/*
This program reads the data produced by timedrawing.exe, analyzes and visualizes it
Not nearly finished, just a crude first version, proof of concept
*/

#include <allegro5/allegro.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_font.h>

#include <stdio.h>
#include <windows.h>
#include <cmath>
#include <ctime>
#include <fstream>

#include "Vector2.h"
#include "Polar2.h"
#include "Xiaoline.h"
#include "misc.h"
#include "PenFrame.h"

//#define TRANSFORM_FROM_POLAR_COORDINATES 1

using namespace Emil;
using std::ios;
using std::ifstream;

const int FRAMES_PER_SECOND = 100;
const double SECONDS_BETWEEN_UPDATES = 1.f / FRAMES_PER_SECOND;
const int inputGameW = 1280; const int inputGameH = 1024;
const int gameW = 1280; const int gameH = 1024;
//const int gameW = 1440; const int gameH = 900;
//const int gameW = 1920; const int gameH = 1280;
const int maxPressureLevel = 1023;

float zoom = 1;
#ifdef TRANSFORM_FROM_POLAR_COORDINATES
	Vector2 pos(gameW/4,gameH/4);
	Vector2 crop(gameW/2,gameH/2);
#else
	Vector2 pos(gameW/2,gameH/2);
	Vector2 crop(gameW,gameH);
#endif

int winsize = 4000;
float winEndColor;


float CalcDecayConstantFromWinsizeInSec(float winsizeInSeconds)
{
	return -log(1.f / 256.f) / winsizeInSeconds;
}

inline Polar2 brainFromRetina(const Vector2 r)
{
	Polar2 b;
	b.a = std::atan2(r.y, r.x);
	// atan2 Return Value
	// Principal arc tangent of y/x, in the interval [-pi,+pi] radians.
	// But we want it in the interval [0,2pi]
	if (b.a < 0)
		b.a += (M_PI*2);

	b.l = std::sqrt(r.x * r.x + r.y * r.y);
	//if (b.y < 0)
	//	b.y = -b.y;
	//if (b.y >= worldW)

	// it's not just a cartesian-to-polar transform, we also want a logarithmic scale
	b.l = log(b.l + 1);

	//const int circleR = circleW/2;
	//const float worldXMultiplier = (worldW / (M_PI*2));
	//const float worldYMultiplier = (worldW / log(circleR));
	//b.a *= worldXMultiplier;
	//b.l *= worldYMultiplier;

	return b;
}

inline Vector2 retinaFromBrain(const Polar2 b)
{
	// reverse the logarithmic scale transform
	float l = exp(b.l)-1;

	Vector2 r;
	r.x = cos(b.a) * l;
	r.y = sin(b.a) * l;
	return r;
}

inline Vector2 brainFromRetina2(Vector2 r)
{
	// preconditions:
	// 0 <= x < crop.x
	// 0 <= y < crop.y
	Polar2 b = brainFromRetina(r);
	Vector2 v;
	v.x = b.a / (M_PI*2) * (gameW/2);
	v.y = b.l * (gameH/8.f);
	return v;
}

inline Vector2 retinaFromBrain2(Vector2 v)
{
	// preconditions:
	// 0 <= v.x < gameW/2
	// 0 <= v.y < gameH
	Polar2 b;
	b.a = v.x * (M_PI*2) / (gameW/2);
	b.l = v.y / (gameH/8.f);
	Vector2 r = retinaFromBrain(b);
	return r;
}

inline Vector2 brainFromRetina3(Vector2 r)
{
	r -= pos;
	Vector2 v = brainFromRetina2(r);
	//v.x += gameW/2;
	return v;
}

inline Vector2 retinaFromBrain3(int x, int y)
{
	Vector2 r = retinaFromBrain2(Vector2(x,y));
	r += pos;
	return r;
}

/*
inline ALLEGRO_COLOR operator+(ALLEGRO_COLOR lhs, const ALLEGRO_COLOR& rhs)
{
	al_unmap_rgb
	return lhs;
}
*/

// When t = 0 returns from.
// When t = 1 returns to.
// When t = 0.5 returns the point midway between from and to.
inline ALLEGRO_COLOR Lerp(ALLEGRO_COLOR from, ALLEGRO_COLOR to, float t)
{
	/*
	unsigned char r0, g0, b0;
	unsigned char r1, g1, b1;
	al_unmap_rgb(from, &r0, &g0, &b0);
	al_unmap_rgb(to, &r1, &g1, &b1);
	unsigned char r, g, b;
	r = (r0 * (1-t)) + (r1 * t);
	g = (g0 * (1-t)) + (g1 * t);
	b = (b0 * (1-t)) + (b1 * t);
	return al_map_rgb(r, g, b);
	*/
	// floating-point actually seems faster on my current computer. haven't measured it though.
	float r0, g0, b0;
	float r1, g1, b1;
	al_unmap_rgb_f(from, &r0, &g0, &b0);
	al_unmap_rgb_f(to, &r1, &g1, &b1);
	float r, g, b;
	r = (r0 * (1-t)) + (r1 * t);
	g = (g0 * (1-t)) + (g1 * t);
	b = (b0 * (1-t)) + (b1 * t);
	return al_map_rgb_f(r, g, b);
}


inline ALLEGRO_COLOR get_pixel_bilerped(ALLEGRO_BITMAP* bitmap, float rx, float ry)
{
	// bilinear interpolation
	int x0 = (int)rx;
	int y0 = (int)ry;
	int x1 = x0+1;
	int y1 = y0+1;
	ALLEGRO_COLOR c00 = al_get_pixel(bitmap, x0, y0);
	ALLEGRO_COLOR c10 = al_get_pixel(bitmap, x1, y0);
	ALLEGRO_COLOR c01 = al_get_pixel(bitmap, x0, y1);
	ALLEGRO_COLOR c11 = al_get_pixel(bitmap, x1, y1);
	float xratio = rx - x0;
	float yratio = ry - y0;
	ALLEGRO_COLOR c0 = Lerp(c00, c10, xratio);
	ALLEGRO_COLOR c1 = Lerp(c01, c11, xratio);
	ALLEGRO_COLOR c = Lerp(c0, c1, yratio);
	return c;
}


// raises the luminance of dark colors (like pure blue) by mixing them with white
inline void MaybeMakeBrighter(const float minLuma, float& r, float& g, float& b)
{
	float luma = (r*0.2126f + g*0.7152f + b*0.0722);
	if (luma < minLuma)
	{
		// Hmm, much easier math to lerp towards black:
		// r*a*0.21 + g*a*0.71 + b*a*0.08 = constant, where 0<a<1
		// a*(r*0.21 + g*0.71 + b*0.08) = constant
		// a = constant / (r*0.21 + g*0.71 + b*0.08)
		// So let's just invert the colors, lerp towards black, then invert again.
		float maxLuma = 1 - minLuma;
		r = 1-r;
		g = 1-g;
		b = 1-b;
		luma = (r*0.2126f + g*0.7152f + b*0.0722);
		float scale = maxLuma / luma;
		r *= scale;
		g *= scale;
		b *= scale;
		r = 1-r;
		g = 1-g;
		b = 1-b;
	}
}


inline double linear_to_sRGB(double y)
{
	if (y <= 0.0031308)
		return 12.92 * y;
	return 1.055 * pow(y, 1/2.4) - 0.055;
}


// precondition: 0 <= s <= smax
inline void mapRainbow(float s, const float smax, float& r, float& g, float& b)
{
	float step = smax / 6.0f;
	if (s < step)
	{
		float ratio = s / step;
		r = 1;
		g = ratio;
		b = 0;
		return;
	}
	s -= step;
	if (s < step)
	{
		float ratio = s / step;
		r = 1-ratio;
		g = 1;
		b = 0;
		return;
	}
	s -= step;
	if (s < step)
	{
		float ratio = s / step;
		r = 0;
		g = 1;
		b = ratio;
		return;
	}
	s -= step;
	if (s < step)
	{
		float ratio = s / step;
		r = 0;
		g = 1-ratio;
		b = 1;
		return;
	}
	s -= step;
	if (s < step)
	{
		float ratio = s / step;
		r = ratio;
		g = 0;
		b = 1;
		return;
	}
	s -= step;
	if (s < step)
	{
		float ratio = s / step;
		r = 1;
		g = 0;
		b = 1-ratio;
		return;
	}
}


void PolarDrawToVirscr(int x, int y, const bool horizontal, float fraction, void* pObject) // used for callback by Xiaoline
{
	if (x < 0 || x >= gameW || y < 0 || y >= gameH)
		return;

	// draws two pixels
	// horizontal=true means we should draw (x;y) and (x+1;y)
	// horizontal=false means we should draw (x;y) and (x;y+1)

	// let's pretend we have a fixed amount of ink
	// fraction=0 means all the ink should go in the first pixel
	// fraction=0.5 means they should divide the ink equally
	// fraction=1 means all the ink should go in the second pixel

	// and the amount of ink we have is given by 'alpha'
	float alpha, rainbowRadian;
	{
		Polar2 p = *((Polar2*)pObject);
		alpha = p.l;
		rainbowRadian = p.a;
	}

	const float minLuma = 0.4f;

	float r, g, b;
	mapRainbow(rainbowRadian, M_PI*2, r, g, b);
	MaybeMakeBrighter(minLuma, r, g, b);

	// TODO: check if per-component gamma correction looks better
	// r = linear_to_sRGB(r);
	// g = linear_to_sRGB(g);
	// b = linear_to_sRGB(b);
	// but this must be done after alpha blending, not before...

	ALLEGRO_COLOR c = al_map_rgba_f(r, g, b, alpha*(1.0f - fraction));
	al_draw_pixel(x, y, c);

	if (horizontal)
		x++;
	else
		y++;

	if (x >= gameW || y >= gameH)
		return;

	mapRainbow(rainbowRadian, M_PI*2, r, g, b);
	MaybeMakeBrighter(minLuma, r, g, b);
	c = al_map_rgba_f(r, g, b, alpha*fraction);
	al_draw_pixel(x, y, c);
}


inline void DrawPenFrame_v2(PenFrame_v2 currPF, PenFrame_v2 prevPF, float iWinEndSec, int& dirty, int decayMethod, int brainAOffset)
{
	// don't draw a line when the pen is up
	//if (currPF.penpressure > 0)
	// hmm, this ALWAYS happens
	// there are no frames with pressure=0 becuase i didn't write them into the file, i skipped them
	// i need to check another way:
	if (currPF.index - prevPF.index == 1)
	{
		Vector2 currR(currPF.penx * zoom, currPF.peny * zoom);
		Vector2 prevR(prevPF.penx * zoom, prevPF.peny * zoom);

		currPF.penx = currPF.penx * zoom + pos.x;
		currPF.peny = currPF.peny * zoom + pos.y;
		prevPF.penx = prevPF.penx * zoom + pos.x;
		prevPF.peny = prevPF.peny * zoom + pos.y;

		float ageInMs = (iWinEndSec*1000 - currPF.wacomtime); // 1000 because iWinEndSec is in seconds, wacomtime is in milliseconds
		float ageInS = ageInMs / 1000.f;

		float alphaDueToAge;
		if (decayMethod == 1)
		{
			// linear decay
			alphaDueToAge = std::max(0.f, 1 - (ageInS / (float)winsize));
		}
		else if (decayMethod == 2)
		{
			// exponential decay
			float decayConstant = CalcDecayConstantFromWinsizeInSec(winsize); // TODO: could be calculated outside the loop, for speed
			alphaDueToAge = 1 * exp(-decayConstant*ageInS);
		}
		else if (decayMethod == 3)
		{
			// symmetric linear attack and decay
			float halfWinsize = winsize/2.f;
			float ageInMs2 = fabs((iWinEndSec-halfWinsize)*1000 - currPF.wacomtime);
			float ageInS2 = ageInMs2 / 1000.f;
			alphaDueToAge = std::max(0.f, 1 - (ageInS2 / halfWinsize));
		}
		else if (decayMethod == 4)
		{
			// symmetric exponential attack and decay
			float halfWinsize = winsize/2.f;
			float ageInMs2 = fabs((iWinEndSec-halfWinsize)*1000 - currPF.wacomtime);
			float ageInS2 = ageInMs2 / 1000.f;
			float decayConstant = CalcDecayConstantFromWinsizeInSec(halfWinsize);
			alphaDueToAge = 1 * exp(-decayConstant*ageInS2);
		}
		// TODO: try other decay methods. i think a sine hill would look nice

		//float alphaDueToPressure = (float)currPF.penpressure/maxPressureLevel; // too hard to see the really weak strokes :(
		//float alphaDueToPressure = 1 - ((1-(float)currPF.penpressure/maxPressureLevel) * 0.8); // easier :)

		//float alpha = (alphaDueToPressure + alphaDueToAge) / 2;
		//float alpha = alphaDueToPressure * alphaDueToAge;
		//float alpha = std::min(alphaDueToPressure, alphaDueToAge);
		float alpha = alphaDueToAge;

		// we want a repeating rainbow for colors
		// one rainbow should fit exactly into our "window"
		// so calculate how many window lengths into the data stream we are
		// the answer will be a decimal number, something like 10.75
		//float remainder = ColorForPositionInSec(currPF.wacomtime / 1000);
		//float scaledTime = (currPF.wacomtime) / ((float)winsize * 1000.f);
		// keep only the amount after the dot, for example 0.75
		//float remainder = fmod(scaledTime, 1);
		float youthInMs = (long)winsize * 1000 - ageInMs;

		float youthInWinsizes = youthInMs / (winsize * 1000.f);
		float remainder = fmod(youthInWinsizes + winEndColor, 1);
		Polar2 p(alpha, remainder*(M_PI*2));
		Xiaoline(currPF.penx, currPF.peny, prevPF.penx, prevPF.peny, PolarDrawToVirscr, &p); // draws to ALLEGRO_BITMAP* virscr
		if (zoom > 1)
		{
			// draw a thicker line
			Xiaoline(currPF.penx, currPF.peny+1, prevPF.penx, prevPF.peny+1, PolarDrawToVirscr, &p);
			Xiaoline(currPF.penx+1, currPF.peny, prevPF.penx+1, prevPF.peny, PolarDrawToVirscr, &p);
		}

		#ifdef TRANSFORM_FROM_POLAR_COORDINATES
			// coordinate transform
			Vector2 currB = brainFromRetina2(currR);
			Vector2 prevB = brainFromRetina2(prevR);
			currB.x = fmod(currB.x + brainAOffset, gameW/2) + (gameW/2);
			prevB.x = fmod(prevB.x + brainAOffset, gameW/2) + (gameW/2);
			if (fabs(currB.x - prevB.x) < gameW/4)
			{
				Xiaoline(currB.x, currB.y, prevB.x, prevB.y, PolarDrawToVirscr, &p);
			}
			// else it should wrap around the edge
		#endif

		dirty = 1;
	}
}


inline float ColorForPositionInSec(float s)
{
	// we want a repeating rainbow for colors
	// one rainbow should fit exactly into our "window"
	// so calculate how many window lengths into the data stream we are
	// the answer will be a decimal number, something like 10.75
	float scaledTime = s / (float)winsize;
	// keep only the amount after the dot, for example 0.75
	float remainder = fmod(scaledTime, 1);
	return remainder;
}


void LoadTestImageFromDisk(unsigned short iCurrTestImage, char* testImageDir, Vector2 pos, Vector2 crop, float zoom)
{
	char* fullFilePath = new char[strlen(testImageDir) + strlen("testimage_00.bmp") + 3];
	sprintf(fullFilePath, "%stestimage_%02d.bmp", testImageDir, iCurrTestImage);
	ALLEGRO_BITMAP* testimage = al_load_bitmap(fullFilePath);
	delete[] fullFilePath;
	if (testimage)
	{
		// TODO: doublecheck this math, i'm not sure i got it right
		Vector2 swh(inputGameW, inputGameH);
		Vector2 sxy = swh/2;
		swh /= zoom;
		sxy -= swh/2;
		Vector2 d = pos - crop/2;
		al_draw_scaled_bitmap(testimage, sxy.x,sxy.y, swh.x,swh.y, d.x,d.y, crop.x,crop.y, 0);
		al_destroy_bitmap(testimage);
	}
}


int AnalyzeVersion2(unsigned int nBytes, ifstream& fin, ALLEGRO_EVENT_QUEUE* event_queue, ALLEGRO_DISPLAY* display, ALLEGRO_BITMAP* virscr, ALLEGRO_FONT *f1)
{
	ALLEGRO_EVENT al_event;

	// read the file stream into the penFrame[] array

	unsigned int nFrames = nBytes / sizeof(PenFrame_v2);
	printf("nFrames=%u\n", nFrames);
	if (nFrames == 0)
		return 1;
	PenFrame_v2* penFrame = new PenFrame_v2[nFrames];
	if (!penFrame)
	{
		printf("new PenFrame_v2[%u]; failed\n", nFrames);
		return 1;
	}
	for (unsigned long i = 0; i < nFrames; i++)
	{
		fin.read((char*)&(penFrame[i]), sizeof(PenFrame_v2));
		if (!fin.good())
		{
			printf("!fin.good()\n");
			break;
		}
	}
	printf("done\n");
	fin.close();

	MessageBeep(MB_ICONEXCLAMATION);

	// measure how long time each test image was active
	unsigned short prevTestimage = penFrame[0].testimage;
	unsigned long msWhenTestimageAppeared = penFrame[0].wacomtime;
	for (unsigned long i = 0; i < nFrames; i++)
	{
		if (prevTestimage != penFrame[i].testimage)
		{
			unsigned long msTestImageWasDisplayed = penFrame[i].wacomtime - msWhenTestimageAppeared;
			printf("showed testimage %hu for %lu minutes\n", prevTestimage, msTestImageWasDisplayed/(1000*60));

			prevTestimage = penFrame[i].testimage;
			msWhenTestimageAppeared = penFrame[i].wacomtime;
		}
	}


	// colors and drawing surfaces

	ALLEGRO_COLOR foregrColor;
	{
		const int foreground_r = 48;
		const int foreground_g = 48;
		const int foreground_b = 48;
		foregrColor = al_map_rgb(foreground_r, foreground_g, foreground_b);
	}
	ALLEGRO_COLOR backgrColor = al_map_rgb(0, 0, 0);



	// measure file length, and fix position&time offsets

	float fileLengthInSeconds;
	{
		short maxpenx = 0;
		short maxpeny = 0;
		unsigned long firstWacomTime = penFrame[0].wacomtime;
		unsigned long lastWacomTime = penFrame[nFrames-1].wacomtime;
		fileLengthInSeconds = (lastWacomTime-firstWacomTime)/1000.f;
		for (unsigned long i = 0; i < nFrames; i++)
		{
			penFrame[i].wacomtime -= firstWacomTime;
			penFrame[i].penx = penFrame[i].penx - inputGameW/2;
			penFrame[i].peny = penFrame[i].peny - inputGameH/2;
			if (penFrame[i].penpressure > 0)
			{
				maxpenx = std::max(maxpenx, (short)abs(penFrame[i].penx));
				maxpeny = std::max(maxpeny, (short)abs(penFrame[i].peny));
			}
		}
		//zoom = std::min((gameW/2) / maxpenx, (gameH/2) / maxpeny); // zoom in as far as possible on this screen
	}


	#ifdef TRANSFORM_FROM_POLAR_COORDINATES
		// make a lookup table for faster translation between retina and brain coordinates
		Vector2* retinaFromBrainLut = new Vector2[(gameW/2) * gameH];
		for (int y = 0; y < gameH; y++)
		{
			for (int x = 0; x < gameW/2; x++)
			{
				Vector2 r = retinaFromBrain3(x, y);
				if (r.x >= 0 && r.x < crop.x && r.y >= 0 && r.y < gameH)
				{
				}
				else
				{
					r = Vector2(0, 0);
				}
				retinaFromBrainLut[y * (gameW/2) + x] = r;
			}
		}
	#endif



	winsize = 25; // how many seconds worth of data do we show at once?
	winEndColor = ColorForPositionInSec(winsize); // only used to keep the same colors when we make the window smaller or larger
	float iWinStartSec = 0;
	int iWinStart = 0;
	int iWinEnd = 0;

	unsigned short iPrevTestImage = -1;
	ALLEGRO_BITMAP* bmpTestImage = al_create_bitmap(gameW, gameH);
	if (!bmpTestImage)
	{
		printf("bmpTestImage = al_create_bitmap() failed\n");
		return 1;
	}
	char testImageDir[] = "testimages/"; // relative, only works from the right working dir
	//char testImageDir[] = "e:/gamedev/inwork/open_eye_visuals/timedrawing/testimages/"; // absolute, always works on my computer

	int iPrevFrame = 0; // previous = the last drawn penframe from the previous lap in the main loop
	int dirty = 1;
	int decayMethod = 1;
	float secPerStep = 0.2f;
	float secPerStepFastForward = 2;
	int brainAOffset = 0;

	// -1=just released
	//  0=not held down
	//  1=just pressed for the first time
	//  2=held down
	int spaceButtonState = 0;

	Vector2 prevmouse;
	ALLEGRO_MOUSE_STATE mousestate;
	al_get_mouse_state(&mousestate);

	ALLEGRO_KEYBOARD_STATE key_state;
	int gameRunning = 1;
	double timePrev = al_get_time();

	while (gameRunning)
	{
		// simple timing: lock the main loop to a fixed speed
		// slower computers will get slowdowns, faster computers will al_rest to use less cpu
		// but al_rest has quite bad accuracy, so this code shouldn't be used in any serious project

		double timeNow = al_get_time();
		//printf("timeNow=%f\n", timeNow);
		double deltaTime = timeNow - timePrev;
		//printf("deltaTime=%f\n", deltaTime);
		double sleeptime = SECONDS_BETWEEN_UPDATES - deltaTime;
		//printf("sleeptime=%f\n", sleeptime);
		//timePrev = timeNow;
		if (sleeptime > 0)
		{
			al_rest(sleeptime);
			//printf("sleep error=%f\n", ((timeNow + sleeptime) - al_get_time()));
			//timePrev += sleeptime;
		}
		float fpsWithoutRest = 1 / deltaTime;
		deltaTime = al_get_time() - timePrev;
		float fpsWithRest = 1 / deltaTime;
		//printf("without rest, fps would be =%f\n", fpsWithoutRest);
		//printf("with rest, fps is =%f\n", fpsWithRest);

		timePrev = al_get_time();


		float iWinEndSec = iWinStartSec + winsize;
		float iWinEndSecBefore = iWinEndSec; // before we modify either iWinStartSec or winsize


		if (al_get_next_event(event_queue, &al_event))
		{
			if (al_event.type == ALLEGRO_EVENT_DISPLAY_CLOSE)
				gameRunning = 0;
		}

		al_get_keyboard_state(&key_state);
		if (al_key_down(&key_state, ALLEGRO_KEY_ESCAPE))
		{
			gameRunning = 0;
		}

		if (al_key_down(&key_state, ALLEGRO_KEY_SPACE))
		{
			if (spaceButtonState < 1)
				spaceButtonState = 1;
			else
				spaceButtonState = 2;
		}
		else
		{
			if (spaceButtonState > 0)
				spaceButtonState = -1;
			else
				spaceButtonState = 0;
		}


		if (al_key_down(&key_state, ALLEGRO_KEY_I))
		{
			dirty = 2;
			if (iWinStartSec-1 > 0)
			{
				winsize += 1;
				iWinStartSec -= 1;
			}
			else
			{
				iWinEndSec = iWinStartSec + winsize;
				if (iWinEndSec+1 < fileLengthInSeconds)
				{
					winsize += 1;
				}
			}
		}
		if (al_key_down(&key_state, ALLEGRO_KEY_O))
		{
			dirty = 2;
			if (winsize-1 > 0)
			{
				winsize -= 1;
				iWinStartSec += 1;
			}
		}

		if (al_key_down(&key_state, ALLEGRO_KEY_A))
		{
			if (zoom < 1 && zoom * 1.1 > 1)
				zoom = 1;
			zoom *= 1.1;
			dirty = 3;
		}
		if (al_key_down(&key_state, ALLEGRO_KEY_S))
		{
			if (zoom > 1 && zoom / 1.1 < 1)
				zoom = 1;
			zoom /= 1.1;
			//if (zoom < 1)
			//	zoom = 1;
			dirty = 3;
		}

		if (al_key_down(&key_state, ALLEGRO_KEY_UP))
		{
			dirty = 1; // means that we need to redraw the screen, but only a little: draw only the new colored lines, then fade them
			if (al_key_down(&key_state, ALLEGRO_KEY_LSHIFT)) // hold down Shift to fast-forward
				iWinStartSec += secPerStepFastForward;
			else
				iWinStartSec += secPerStep;
            iWinEndSec = iWinStartSec + winsize;
            if (iWinEndSec > fileLengthInSeconds)
            {
				iWinEndSec = fileLengthInSeconds;
				iWinStartSec = iWinEndSec - winsize;
            }
		}
		if (al_key_down(&key_state, ALLEGRO_KEY_DOWN))
		{
			dirty = 2;
			if (al_key_down(&key_state, ALLEGRO_KEY_LSHIFT))
				iWinStartSec -= secPerStepFastForward;
			else
				iWinStartSec -= secPerStep;
            if (iWinStartSec < 0)
				iWinStartSec = 0;
		}

		if (al_key_down(&key_state, ALLEGRO_KEY_LEFT))
		{
			dirty = 3;
            brainAOffset -= 16;
            if (brainAOffset < 0)
				brainAOffset += (gameW/2);
		}
		if (al_key_down(&key_state, ALLEGRO_KEY_RIGHT))
		{
			dirty = 3;
            brainAOffset += 16;
            if (brainAOffset > (gameW/2))
				brainAOffset -= (gameW/2);
		}

		if (al_key_down(&key_state, ALLEGRO_KEY_1))
			decayMethod = 1;
		if (al_key_down(&key_state, ALLEGRO_KEY_2))
			decayMethod = 2;
		if (al_key_down(&key_state, ALLEGRO_KEY_3))
			decayMethod = 3;
		if (al_key_down(&key_state, ALLEGRO_KEY_4))
			decayMethod = 4;

		if (spaceButtonState == 1)
		{
			al_save_bitmap("printscreen.bmp", virscr);
		}

		iWinEndSec = iWinStartSec + winsize;

		if (iWinEndSecBefore != iWinEndSec)
		{
			winEndColor += ColorForPositionInSec(iWinEndSec - iWinEndSecBefore);
			// otherwise the newest line would always be red.
			// this calculation makes sure each line keeps their color.
		}

		if (dirty)
		{
			// we want to show all penFrames in the range between iWinStartSec and iWinEndSec
			// which penFrames is that? based on the timestamps, find the corresponding array indexes!
			while (iWinStart > 0         && iWinStartSec < penFrame[iWinStart].wacomtime/1000.f)
				iWinStart--;
			while (iWinStart < nFrames-1 && iWinStartSec > penFrame[iWinStart].wacomtime/1000.f)
				iWinStart++;
			while (iWinEnd <   nFrames-1 && iWinEndSec   > penFrame[iWinEnd  ].wacomtime/1000.f)
				iWinEnd++;
			while (iWinEnd   > 0         && iWinEndSec   < penFrame[iWinEnd  ].wacomtime/1000.f)
				iWinEnd--;

			unsigned short iCurrTestImage = penFrame[iWinEnd].testimage;
			if (iCurrTestImage != iPrevTestImage || dirty == 3)
			{
				//DrawTestImage(iCurrTestImage, foregrColor, backgrColor, pos, crop, zoom, bmpTestImage);

				al_set_target_bitmap(bmpTestImage);

				LoadTestImageFromDisk(iCurrTestImage, testImageDir, pos, crop, zoom);

				al_lock_bitmap(bmpTestImage, ALLEGRO_PIXEL_FORMAT_ANY, ALLEGRO_LOCK_READWRITE);

				#ifdef TRANSFORM_FROM_POLAR_COORDINATES
					// do coordiante transform of the test image
					for (int y = 0; y < gameH; y++)
					{
						for (int x = gameW/2; x < gameW; x++)
						{
							int nx = (x - brainAOffset) % (gameW/2);
							Vector2 r = retinaFromBrainLut[y * (gameW/2) + nx];
							//Vector2 r = retinaFromBrain3(nx, y);
							//if (r.x >= 0 && r.x < crop.x && r.y >= 0 && r.y < gameH)
							{
								ALLEGRO_COLOR c = get_pixel_bilerped(bmpTestImage, r.x, r.y);
								al_put_pixel(x, y, c);
							}
						}
					}
				#endif
				al_unlock_bitmap(bmpTestImage);
			}

			al_set_target_bitmap(virscr);
			al_draw_bitmap(bmpTestImage, 0, 0, 0);

			al_put_pixel(pos.x, pos.y, al_map_rgb_f(1,1,1));
			//al_put_pixel(pos.x-1, pos.y, al_map_rgb_f(0,0,0));
			//al_put_pixel(pos.x, pos.y-1, al_map_rgb_f(0,0,0));
			//al_put_pixel(pos.x-1, pos.y-1, al_map_rgb_f(1,1,1));

			al_lock_bitmap(virscr, ALLEGRO_PIXEL_FORMAT_ANY, ALLEGRO_LOCK_READWRITE);
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA);
			{
				PenFrame_v2 prevPF = penFrame[iWinStart];
				for (int iFrame = iWinStart+1; iFrame <= iWinEnd; iFrame++)
				{
					if (iFrame >= 0 && iFrame < nFrames)
					{
						PenFrame_v2 currPF = penFrame[iFrame];
						DrawPenFrame_v2(currPF, prevPF, iWinEndSec, dirty, decayMethod, brainAOffset);
						prevPF = currPF;
					}
				}
				iPrevFrame = iWinEnd;
			}
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);

			al_unlock_bitmap(virscr);
			al_set_target_bitmap(al_get_backbuffer(display));
			al_draw_bitmap(virscr, 0, 0, 0);

			{
				float x = secPerStep / deltaTime;
				al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,48, 0, "which means we are playing back at %dx original speed", (int)x);
			}

			iPrevTestImage = iCurrTestImage;

			dirty = 0;
		}


		al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,8, 0, "frame between %d and %d (out of %d) (change with arrow keys up / down)", iWinStart, iWinEnd, nFrames);

		{
			int seconds0 = iWinStartSec;
			unsigned long minutes0 = (unsigned long)seconds0/60;
			seconds0 = seconds0 - (minutes0*60);
			int seconds1 = iWinEndSec;
			unsigned long minutes1 = (unsigned long)seconds1/60;
			seconds1 = seconds1 - (minutes1*60);
			al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,0, 0, "time between %lu min %d sec and %lu min %d sec", minutes0, seconds0, minutes1, seconds1);
		}

		{
			int seconds = fileLengthInSeconds;
			unsigned long minutes = (unsigned long)seconds/60;
			seconds = seconds - (minutes*60);
			al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 400,0, 0, "/ %lu min %d sec", minutes, seconds);
		}

		al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,24, 0, "winsize %d sec (change with keys I / O)", winsize);
		al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,32, 0, "without rest, fps would be %f", fpsWithoutRest);
		al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,40, 0, "with rest, fps is %f", fpsWithRest);

		if (decayMethod == 1)
			al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,56, 0, "linear decay (change with keys 1-4)");
		else if (decayMethod == 2)
			al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,56, 0, "exponential decay (change with keys 1-4)");
		else if (decayMethod == 3)
			al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,56, 0, "symmetric linear attack and decay (change with keys 1-4)");
		else if (decayMethod == 4)
			al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,56, 0, "symmetric exponential attack and decay (change with keys 1-4)");

		al_draw_textf(f1, al_map_rgb_f(1, 1, 1), 0,64, 0, "zoom %f (change with keys A / S)", zoom);

		al_flip_display();

	}

	return 0;
}



int main(int argc, char* argv[])
{
	if (!al_init())
	{
		printf("al_init() failed\n");
		return 1;
	}

	if (!al_install_keyboard())
	{
		printf("al_install_keyboard() failed\n");
		return 1;
	}

	if (!al_install_mouse())
	{
		printf("al_install_mouse() failed\n");
		return 1;
	}

	//al_set_new_display_flags(ALLEGRO_FULLSCREEN);
	//al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW | ALLEGRO_SINGLE_BUFFER);
	al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW);

	ALLEGRO_DISPLAY* display = al_create_display(gameW, gameH);
	if (!display)
	{
		printf("al_create_display(%d,%d) failed\n", gameW, gameH);
		return 1;
	}

	ALLEGRO_EVENT_QUEUE* event_queue = al_create_event_queue();
	if (!event_queue)
	{
		printf("al_create_event_queue() failed\n");
		return 1;
	}
	al_register_event_source(event_queue, al_get_display_event_source(display));

	if (!al_init_primitives_addon())
	{
		printf("al_init_primitives_addon() failed\n");
		return 1;
	}

	if (!al_init_image_addon())
	{
		printf("al_init_image_addon() failed\n");
		return 1;
	}

	al_init_font_addon();

    ALLEGRO_FONT *f1 = al_load_font("a4_font.tga", 16, 0);
    if (!f1)
	{
		printf("al_load_font() failed\n");
		return 1;
	}

	al_clear_to_color(al_map_rgb(0, 0, 0));

	srand((int)(al_get_time()*10000));

	al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP);

	ALLEGRO_BITMAP* virscr = al_create_bitmap(gameW, gameH);
	if (!virscr)
	{
		printf("al_create_bitmap() failed\n");
		return 1;
	}


	// load the raw data from the file, so we can analyze it!

	char* filename;
	if (argc == 1)
	{
		filename = "timedrawing_2013-11-30.oevlog";
	}
	else if (argc == 2)
	{
		filename = argv[1];
	}
	else
	{
		printf("too many commandline arguments\n");
		return 1;
	}

	ifstream fin;

	printf("filename=%s\n", filename);
	fin.open(filename, ios::in | ios::binary);
	if (fin.fail())
	{
		printf("fin.open failed\n");
		return 1;
	}

	fin.seekg(0, ios::end);
	unsigned int nBytes = fin.tellg();
	printf("nBytes=%u\n", nBytes);
	fin.seekg(0, ios::beg);

	// TODO: more checks to see if it's a valid file

	int returnCode = AnalyzeVersion2(nBytes, fin, event_queue, display, virscr, f1);

	al_destroy_display(display);

	//system("pause");
	return returnCode;
}

